// tree.js -- code to support the index tree (in archangel.xul).
// $Header: /cvs/archangel/src/archangel/content/tree.js,v 1.10 2003/02/10 08:43:09 marnanel Exp $
//
// Copyright (c) 2002 Thomas Thurman
// thomas@thurman.org.uk
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have be able to view the GNU General Public License at 
// http://www.gnu.org/copyleft/gpl.html ; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

////////////////////////////////////////////////////////////////

var server = 0;   // RGTP connection to the server
var vault = 0;    // RDF file containing all our state
var groggs = 0;   // Our (currently) only server, inside "serverlist"
var backing_store = 0;

var known_sequences = 0;

// hook into Netscape's dialogue boxes.
var promptService =
	Components.classes['@mozilla.org/embedcomp/prompt-service;1'].
	getService().
	QueryInterface(Components.interfaces.nsIPromptService);

////////////////////////////////////////////////////////////////

// Point of entry when the tree is first loaded.

function treeStart() {

	function createVault() {
		var du = new DirUtils();
		vault = new RDFFile(new Dir(du.getMozUserHomeDir()).append("rgtpserv.rdf"));
		if (vault==null) throw 'problem: vault is null';

		// GROGGS (server s1) is hard-coded in this release.
		groggs = vault.addRootBag("urn:servers:s1");

		if (!groggs.getAttribute('host')) {
			groggs.setAttribute('host', 'rgtp-serv.groggs.group.cam.ac.uk');
			groggs.setAttribute('port', 1431);
		}

		vault.flush();
	}

	function connectToServer() {
		server = new RGTPConnection(groggs);
		// The next line seems to indicate that a rethink might be
		// in order. :)
		server.evalAfterNextJob('if(this.valid){this.evalAfterNextJob("populateTree()");this.getIndex()}else{populateTree();}');
		server.open();

		// When populateTree() works for refreshing too, add a call to it here.
		// That way, they get to see the tree while the index is downloading.
	}

	////////////////////////////////////////////////////////
	// So then...
	
	createVault();
	connectToServer();
}

////////////////////////////////////////////////////////////////

ONE_DAY = 60*60*24;
ONE_YEAR = ONE_DAY*365;

function prettyDate(date) {
	var then = (new Date(date*1000).toString());
	var now = (new Date().toString());
	
	if (then.substring(0,16)==now.substring(0,16)) {
		// Then it was today. Only show them the time.
		return then.substring(16,21);
	} else if (then.substring(11,16)==now.substring(11,16)) {
		// This year sometime.
		// (I'd rather make this a year==365 days, not
		// just any date with the same year number.)
		return then.substring(4,10)+', '+then.substring(16,21);
	} else {
		// More than a year.
		return then.substring(4,16);
	}

	throw "impossible (prettyDate)";	
}

function populateTree() {
	// fixme: need provision for clearing away the old nodes

	setStatus('Formatting index');
	stripyMeter();

	var items = vault.getRootBag('urn:servers:s1').getSubNodes();

	known_sequences = [];
	details = [];

	for (var i=0; i<items.length; i++) {
		var itemid = items[i].getSubject();
		itemid = itemid.substring(itemid.length-8);

		var known = items[i].getAttribute('known');

		known_sequences.push(known);
		
		var props = '';

		// "unread" needs to be the first one, to make the string
		// processing easier: it's the only one we change on the fly.

		if (items[i].getAttribute('seen')!=known) props = props + 'unread ';

		var child = items[i].getAttribute('child');
		if (items[i].getAttribute('child')!=null) props = props + 'full ';

		details[known] = [props,
			items[i].getAttribute('subject'),
			itemid,
			prettyDate(items[i].getAttribute('last-date')),
			child==null?items[i].getAttribute('last-by'):'',
			items[i].getAttribute('replies'),
		];
	}

	known_sequences.sort();
	known_sequences.reverse();

	for (var j=0; j<known_sequences.length; j++) {
		addToIndex(details[known_sequences[j]]);
	}

	calmMeter();
}

function catcher(code) {
	try {
		eval(code);
	} catch(e) {
		alert(code+': '+e);
		throw e;
	}
}

var inItemHead = 0;
var itemHeaderBlock = [];
var grogname = 0;
var userid = 0;

var shortHeadForm = /from (.*) \(([^(]*)\) at/;
var longHeadFormUserid = /from ([^ ]*) at/;
var longHeadFormGrogname = /^From (.*)$/;
var initialItemLine = 0;

function maybeSetContinueBlock(fromItemid, relationship, continued, arrowLeft) {
	var familyNode = vault.getNode('urn:servers:s1:'+fromItemid).getAttribute(relationship);
	
	if (familyNode != null)
		setContinueBlock(continued,
			vault.getNode(familyNode).getAttribute('subject'),
			arrowLeft);
}

function treeLine() {
	return document.getElementById('index-tree').currentIndex;
}

function indexClick() {
	var line = treeLine();

	if (line==0) return;
	line = line - 1; // ick

	var itemid = lineNumberToItemid(line);

	clearItemLines();
	
	maybeSetContinueBlock(itemid, 'parent', 'Continued from ', 0);
		
	inItemHead = 0;
	initialItemLine = 1;
	server.evalAfterNextJob('maybeSetContinueBlock("'+itemid+
		'","child","Continued in ",1);markItemAsSeen('+
		line+')');
	server.getItem(itemid);
}

// function doOpenInNewWindow...
// This will need us to store the target of all the HTML building.
// At present, it's hard-coded.

function doOpenInYarrow() {
	var line = treeLine();

	if (line==0) return;
	line = line - 1; // ick

	// When we have a preferences dialogue, this URL will be
	// customisable.
	var URL = 'http://rgtp.thurman.org.uk/gossip/groggs/$ITEM'

	window.open(URL.replace('$ITEM', lineNumberToItemid(line)), '_blank')
}

function doPrint() {
	window.content.print();
}

function lineNumberToItemid(line) {
	return document.
		getElementById('groggs-items').
		childNodes[line].
		firstChild.
		childNodes[1].
		getAttribute('label');
}

function markItemAsSeen(line, noFlush) {
	// This function needs to:
	//    1) Set "seen" of itemid equal to "known"

	var itemNode = vault.getNode('urn:servers:s1:'+lineNumberToItemid(line));
	itemNode.setAttribute('seen',itemNode.getAttribute('known'));
	
	if (!noFlush) vault.flush();
	
	//    2) Reset the "properties" field of the relevant line to
	//	remove the "unread" part.
	
	var nodelist = document.getElementById('groggs-items').childNodes[line].firstChild.childNodes;
	
	for (var i=0; i<nodelist.length; i++) {
		var node = nodelist[i];
		var props = node.getAttribute('properties');
		
		if (props.substring(0,6)=='unread')
			node.setAttribute('properties', props.substring(7));
	}
}

function receivedItemLine(line) {

	if (initialItemLine) {
		// should process it somehow, but for now...
		initialItemLine = 0;
		return;
	}

	if (!inItemHead && line[0]=='^') {
		// Start of an item head
		inItemHead = 1;
		grogname = 'Grogname?';
		userid = 'Userid?';
		var postedDate = new Date(1000*('0x'+line.substring(10)));
		itemHeaderBlock = [postedDate.toGMTString()];
	} else if (inItemHead && line=='') {
		itemHeaderBlock.push(userid);
		itemHeaderBlock.push(grogname);
		addAttributionBlock(itemHeaderBlock);
		inItemHead = 0;
	} else if (inItemHead) {
			if (shortHeadForm.test(line)) {
				grogname = RegExp.$1;
				userid = RegExp.$2;
		    } else if (longHeadFormGrogname.test(line))
				grogname = RegExp.$1;
		    else if (longHeadFormUserid.test(line))
				userid = RegExp.$1;
	} else
		addItemLine(line);
}

function exitProgram() {
	window.close();
}

function userSaysYesTo(question) {
	return promptService.confirmEx(window, 'Archangel',
		question,
		promptService.BUTTON_TITLE_NO  * promptService.BUTTON_POS_1 +
		promptService.BUTTON_TITLE_YES * promptService.BUTTON_POS_0,
		null, null, null, null, {value:0})==0
}

function doCatchUp() {
	if (userSaysYesTo('This will mark all items on GROGGS as read.'+
		' Are you sure you want to do that?')) {
		
		setStatus('Marking all items as read');
		stripyMeter();
		
		var nodes = document.getElementById('groggs-items').childNodes;
		
		// call "markItemAsSeen" on all lines
		for (var i=0; i<nodes.length; i++)
			markItemAsSeen(i, 1);

		vault.flush();
		
		calmMeter();
	}
}

function aboutBox() {
	alert('Archangel v0.3.0...');
}
